var GridController = (function(_super) {
	__extends(GridController1, _super);

	function GridController1($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query, CrosstabService,
		$filter, maskEngine, ZhUtil, $window, ApplicationContext, ZhFeatures, eventAggregator, DecodeValueFieldService, MessageService, ZhLogger) {
		var _ref = GridController.__super__.constructor.apply(this, arguments);

		// BEGIN: Initialize
		var self = this;
		eventAggregator.publish("OnGridStartUp", {
			widget: $scope.widget
		});
		// END: Initialize

		// Floating Control defaults
		var floatingControlDefaults = {
			pdfAction: true,
			xlsAction: true,
			csvAction: true,
			customizationAction: {
				filter: true,
				layout: true,
				view: true
			}
		};
		self.updateFloatingControlDefaults(floatingControlDefaults);

		// BEGIN: Inline editable
		var getEditableFields = function(widget) {
			return widget.fields.filter(function(field) {
				return field.inlineEditable;
			});
		};

		var hasEditableFields = function(widget) {
			return getEditableFields(widget).length > 0;
		};


		function changeActionsVisibility(actions) {
			if (actions) {
				actions.forEach(function(action) {
					if (action.showAsAction == 'never') {
						action.showAsAction = 'never_list';
					}
					if (action.showAsAction == 'back') {
						action.showAsAction = 'back_list';
					}
				});
			}
		}

		function _cancelActionClick(s) {
			var widget = this;
			widget.inlineEditing = false;

			if (widget.onCancelInlineEdit) {
				var changedRows = widget.dataSource.data.filter(function(row) {
					return row.changed;
				});
				widget.onCancelInlineEdit({
					data: {
						changedRows: changedRows
					}
				});
			} else {
				_rollbackInlineEdit.apply(this);
				widget.list();
			}
		}

		function _rollbackInlineEdit() {
			var widget = this;
			var editableFields = getEditableFields(widget);
			widget.dataSource.data.forEach(function(row) {
				editableFields.forEach(function(field) {
					if (row.hasOwnProperty('changed_' + field.name)) {
						row[field.name] = row['changed_' + field.name];
						if (row[field.name + "_inline"]) {
							delete row[field.name + "_inline"];
						}
						delete row['changed_' + field.name];
					}
				});
				delete row.changed;
			});
			$timeout(function() {
				$(window).trigger('resize.grid', widget.id);
			}, 0, false);
		}

		function _saveActionClick(s) {
			var widget = this;
			widget.inlineEditing = false;
			var lastInlineinput = widget.lastInlineEditingElement.find('input');
			if (lastInlineinput.is(':focus')) {
				lastInlineinput.blur();
			}

			if (widget.onSaveInlineEdit) {
				var changedRows = widget.dataSource.data.filter(function(row) {
					return row.changed;
				});
				widget.onSaveInlineEdit({
					data: {
						changedRows: changedRows
					}
				});
			} else {
				_commitInlineEdit.apply(this);
			}
		}

		function _commitInlineEdit() {
			var widget = this;
			var editableFields = getEditableFields(widget);

			widget.dataSource.data.forEach(function(row) {
				editableFields.forEach(function(field) {
					if (row.hasOwnProperty('changed_' + field.name)) {
						delete row['changed_' + field.name];
					}
				});
				delete row.changed;
			});
			widget.list();
			$timeout(function() {
				$(window).trigger('resize.grid', widget.id);
			}, 0, false);
		}

		function addInlineEditableCancelAction(widget){
			var cancelAction = Util.extend(new Action(), {
				name: 'cancel',
				isVisible: true,
				isZhuAclElement: true,
				label: ScreenService.i18n('Cancel'),
				showAsAction: 'back_inline_edit',
				hideIcon: true,
				events: [{
					'name': 'ActionEvent',
					'code': 'args.owner.widget.cancelActionClick();'
				}]
			});
			cancelAction.widget = widget;
			widget.actions.push(cancelAction);
			widget.container.actions = widget.container.actions || [];
			widget.container.actions.push(cancelAction);
		}

		function addInlineEditableSaveAction(widget){
			var saveAction = Util.extend(new Action(), {
				name: 'save',
				isVisible: true,
				isZhuAclElement: true,
				label: ScreenService.i18n('Save changes'),
				showAsAction: 'never_inline_edit',
				events: [{
					name: 'ActionEvent',
					code: 'args.owner.widget.saveActionClick();'
				}]
			});
			saveAction.widget = widget;
			widget.actions.push(saveAction);
			widget.container.actions.push(saveAction);
		}

		if (hasEditableFields($scope.widget)) {
			if(!$scope.widget.editableInit){
				changeActionsVisibility($scope.widget.actions);
				changeActionsVisibility($scope.widget.container.actions);
				addInlineEditableCancelAction($scope.widget);
				addInlineEditableSaveAction($scope.widget);
				$scope.widget.editableInit = true;
			}

			$scope.widget.cancelActionClick = $scope.widget.cancelActionClick || _cancelActionClick;
			$scope.widget.saveActionClick = $scope.widget.saveActionClick || _saveActionClick;
			$scope.widget.commitInlineEdit = $scope.widget.commitInlineEdit || _commitInlineEdit;
			$scope.widget.rollbackInlineEdit = $scope.widget.rollbackInlineEdit || _rollbackInlineEdit;

		}

		this.inlineEditableCondition = function(row, column) {
			return Util.isEmptyOrBlank(column.inlineEditableCondition) || (function() {
				var result = false;
				ApplicationContext.registerInstance('row', row || {});

				try {
					result = ZhUtil.evaluate("@" + column.inlineEditableCondition + "@");
				} catch (e) {
					ZhLogger.error("Error while evaluating Inline Editable condition function", e);
				}

				ApplicationContext.removeInstance('row');
				return result;
			})();
		};
		// END: inline editable

		if ($scope.widget.checkRowOnClickColumn) {
			var widget = $scope.widget;
			var toggleSelectedRow = function() {
				widget.toggle(widget.currentRow);
			};
			eventEngine.bindEvent(widget, "WidgetAfterMoveRow", toggleSelectedRow);
		}

		$scope.widget._topRow = 0;
		$scope.widget._hiddenRows = 0;
		$scope.widget._rowsUpdate = false;
		this._rows = [];

		this.getVisibleRowCount = function() {
			var result = [];
			var widget = $scope.widget;

			if (widget._getVisibilityParams) {
				widget._getVisibilityParams();
				if (widget.groupedData) {
					var groupedDataFilteredByClosedStatus = widget.groupedData.filter(function(row) {
						return !self.getClosedStatus(row);
					});
					var dataLength = groupedDataFilteredByClosedStatus.length;
					var rowCount = Math.min(dataLength, widget._visibilityParams.maxVisibleRows);

					result = Util.range(rowCount);
					var lastRow = widget._topRow + rowCount;
					self._rows = groupedDataFilteredByClosedStatus
						.slice(widget._topRow, lastRow);
					widget._hideLoadMore = groupedDataFilteredByClosedStatus.length != lastRow;
					if (widget._hiddenRows !== dataLength - rowCount) {
						widget._hiddenRows = dataLength - rowCount;
						widget._calcScrollAreas();
					}
				}
			}

			return result;
		};
		$scope.getTooltip = function(column){
			return column.expression[0] === '@' ? column.expressionName : column.expression;
		};

		$scope.widget._updateVisibleRows = this._updateVisibleRows = Util.buildDebounceMethod(function() {
			if ($scope.widget.groupedData && $scope.widget.groupedData.length > 0) {
				self._rows = $scope.widget.groupedData
					.filter(function(row) {
						return !self.getClosedStatus(row);
					})
					.slice($scope.widget._topRow, $scope.widget._topRow + self._rows.length);
				if (Util.isMobile()) {
					$scope.widget._calcScrollAreas();
				}
				$scope.widget._rowsUpdate = true;

				templateManager.updateTemplate();
			}
		}, 50);

		this.columnHasCheckedRows = function(fieldWithCheckbox) {
			return fieldWithCheckbox.widget.dataSource.data.some(function(row) {
				return row[fieldWithCheckbox.name] == fieldWithCheckbox.valueToTrue;
			});
		};

		this.onClickCheckboxHeader = function(fieldWithCheckbox, event) {
			changeCheckboxColumnValueOnAllRows(fieldWithCheckbox, !this.columnHasCheckedRows(fieldWithCheckbox));
			event.stopPropagation();
		};

		function changeCheckboxColumnValueOnAllRows(fieldWithCheckbox, value) {
			value = value ? fieldWithCheckbox.valueToTrue : fieldWithCheckbox.valueToFalse;
			var savedCurrentRow = fieldWithCheckbox.widget.currentRow;
			var changedRows = [];
			fieldWithCheckbox.widget.dataSource.data.forEach(function(row) {
				fieldWithCheckbox.widget.currentRow = row;
				var fieldValue = row[fieldWithCheckbox.name];

				if (value != fieldValue) {
					row[fieldWithCheckbox.name] = value;

					if (fieldWithCheckbox.change) {
						fieldWithCheckbox.change();
					}

					changedRows.push(row);
				}
			});
			fieldWithCheckbox.widget.currentRow = savedCurrentRow;

			if (fieldWithCheckbox.onInlineEdit) {
				fieldWithCheckbox.onInlineEdit({
					data: {
						field: fieldWithCheckbox,
						rows: changedRows
					}
				});
			}
		}

		this.onClickCheckboxColumn = function(checkboxRow, fieldWithCheckbox, event) {
			if(!fieldWithCheckbox.isReadOnly()){
				fieldWithCheckbox.widget.inlineEdit();
				var checkboxElement = $(event.target);
				fieldWithCheckbox.widget.lastInlineEditingElement = checkboxElement;
				checkboxElement.addClass('toggling');
				if(!checkboxRow['changed_' + fieldWithCheckbox.name]){
					checkboxRow['changed_' + fieldWithCheckbox.name] = checkboxRow[fieldWithCheckbox.name] || fieldWithCheckbox.valueToFalse;
				}
				toggleCheckboxColumnValueOnRow(checkboxRow, fieldWithCheckbox);
				$timeout(function() {
					$(event.target).removeClass('toggling');
				}, 500, false);
				event.stopPropagation();
			}
		};

		function toggleCheckboxColumnValueOnRow(checkboxRow, fieldWithCheckbox) {
			fieldWithCheckbox.widget.currentRow = checkboxRow;
			var value = checkboxRow[fieldWithCheckbox.name];

			if (value === fieldWithCheckbox.valueToTrue) {
				checkboxRow[fieldWithCheckbox.name] = fieldWithCheckbox.valueToFalse;
			} else {
				checkboxRow[fieldWithCheckbox.name] = fieldWithCheckbox.valueToTrue;
			}

			if (fieldWithCheckbox.change) {
				fieldWithCheckbox.change();
			}
			if (fieldWithCheckbox.onInlineEdit) {
				fieldWithCheckbox.onInlineEdit({
					data: {
						field: fieldWithCheckbox
					}
				});
			}
		}

		this.isChildrenWithCheckbox = function(row) {
			return (row.__groupHeaders && row.__groupHeaders.length > 0 && $scope.widget.showCheckbox);
		};

		this.calcLeftPaddingForChildrenRow = function(row, $index, column) {
			var style = '';
			var inlineEditable = column && column.inlineEditable && column.inlineEditable !== 'checkbox';
			if (row && ($index === 0 || inlineEditable)) {
				var totalOfHeaders = row.__groupHeaders ? row.__groupHeaders.length : 0;
				var checkboxWidth = self.isChildrenWithCheckbox(row) ? 22 : 0;
				var inlineEditablePadding = inlineEditable && !row.hasOwnProperty('changed_'+column.name) ? 6 : 0;
				style = {
					'padding-left': totalOfHeaders * 27 - checkboxWidth + inlineEditablePadding
				};
			}
			return style;
		};

		this.getStripeFooterTemplate = function(widget) {
			return (widget && widget.template) ?
				templateManager.getFrameworkTemplateURL(widget.template) : '';
		};

		this.isInlineEditableCheckboxType = function(field) {
			return field.inlineEditable === 'checkbox';
		};

		this.isInlineEditableInputType = function(field) {
			return field.inlineEditable !== 'checkbox';
		};

		// fields ordering
		$scope.fieldOrder = function(field) {
			return (field.grouped ? '1' : '0') +
				(field.fixed ? '0' : '1') +
				('0000' + field.sequenceInList).substr(-4);
		};

		function getFilteredFields(widget, fieldOrder) {
			var fields = $filter('orderBy')(widget.fields, fieldOrder);
			return fields.filter(function(field) {
				return field.showOnList && field.isVisible && field.aclIsVisible;
			});
		}

		self.widget.updateFilteredFields = function() {
			self.widget._filteredFields = getFilteredFields(self.widget, $scope.fieldOrder);
		};

		self.widget.updateFilteredFields();

		$scope.$watchCollection(function() {
			return self.widget._filteredFields;
		}, function() {
			if (self.widget.redraw) {
				self.widget.redraw(true);
			}
		});

		$scope.$on('zh-scrolled-finished', function() {
			if (self.widget.resizeGridColumns) {
				$timeout(function() {
					self.widget.resizeGridColumns(true);
				}, 100);
			}
		});

		var _widget = self.widget;
		$scope.$on('$destroy', function() {
			delete _widget.updateFilteredFields;
			delete _widget._updateVisibleRows;
			delete _widget._filteredFields;
			delete _widget.cancelActionClick;
			delete _widget.saveActionClick;
		});

		return _ref;
	}

	return GridController1;
})(ListController);

ZeedhiController.controller('GridController', GridController);